
# removeMatchingLines.py
# Daniel Schroeder, 2012-01-28

# Takes a "testing" corpus and the training corpus and produces
# a subset of the testing corpus which does not contain any
# sentence pairs where either sentence is found verbatim in the
# training corpus.

import sys

def lineToTuple(line):
    """Takes a raw line from fileObject.readline() and
    returns a tuple of the words in the sentence"""

    rawSplit = line.strip("\n").split(" ")
    return tuple([a for a in rawSplit if a != ""])

def generateSentenceTuples(filename):
    """Yields up tuples for subsequent sentence lines in
    filename, representing each sentence as a tuple of the
    words in the sentence"""

    fObj = open(filename, 'r')

    line = fObj.readline()

    while len(line) != 0:
        lineTuple = lineToTuple(line)

        #print "line tuple:", lineTuple
        yield lineTuple

        line = fObj.readline()

    fObj.close()

def fileToTupleSet(filename):
    """Reads in a file filename of sentences and returns a set
    containing, for each sentence in the file, a tuple of the
    words in the sentence"""

    sentenceSet = set()

    for lineTuple in generateSentenceTuples(filename):
        sentenceSet.add(lineTuple)

    return sentenceSet

def createSubsets(oneInName, twoInName, oneBlacklist, twoBlacklist, oneOutName, twoOutName):
    """Reads in the corpus stored in the files oneInName and
    twoInName and outputs a subcorpus to oneOutName and twoOutName
    containing the sentence pairs where the tuples for the sentences
    do not show up in the sets oneBlacklist or twoBlacklist"""

    oneInObj = open(oneInName, 'r')
    twoInObj = open(twoInName, 'r')

    oneOutObj = open(oneOutName, 'w')
    twoOutObj = open(twoOutName, 'w')

    oneLine = oneInObj.readline()
    twoLine = twoInObj.readline()

    while len(oneLine) != 0 and len(twoLine) != 0:
        oneTuple = lineToTuple(oneLine)
        twoTuple = lineToTuple(twoLine)

        if oneTuple not in oneBlacklist and twoTuple not in twoBlacklist:
            # then write them out
            oneOutObj.write(oneLine)
            twoOutObj.write(twoLine)

        oneLine = oneInObj.readline()
        twoLine = twoInObj.readline()

    assert len(oneLine) == 0 and len(twoLine) == 0 # corpus file lengths match

    oneOutObj.close()
    twoOutObj.close()

    oneInObj.close()
    twoInObj.close()

def main():
    print "Hello world!"

    if len(sys.argv) != 7:
        print "usage: %s oneInName twoInName oneBlackName twoBlackName oneOutName twoOutName" % sys.argv[0]
        return

    oneBlacklist = fileToTupleSet(sys.argv[3])
    twoBlacklist = fileToTupleSet(sys.argv[4])

    createSubsets(sys.argv[1], sys.argv[2], oneBlacklist, twoBlacklist,\
        sys.argv[5], sys.argv[6])

if __name__ == "__main__":
   main()
