
# partitionOnKnownness.py
# Daniel Schroeder and Andy Bouchard, 2012-02-19

# Processes a testing corpus to partition it into two subcorpora,
# one which contains no sentences with unknown words (in either
# language) and the other for which all sentences contain unknown
# words.

import sys

def lineToTuple(line):
    """Takes a raw line from fileObject.readline() and
    returns a tuple of the words in the sentence"""

    rawSplit = line.strip("\n").split(" ")
    return tuple([a for a in rawSplit if a != ""])

def generateSentenceTuples(filename):
    """Yields up tuples for subsequent sentence lines in
    filename, representing each sentence as a tuple of the
    words in the sentence"""

    fObj = open(filename, 'r')

    line = fObj.readline()

    while len(line) != 0:
        lineTuple = lineToTuple(line)

        #print "line tuple:", lineTuple
        yield lineTuple

        line = fObj.readline()

    fObj.close()

def fileToWordSet(filename):
    wordSet = set()

    for lineTuple in generateSentenceTuples(filename):
        for word in lineTuple:
            wordSet.add(word)

    return wordSet

def createSubsets(oneInName, twoInName, oneKnownWords, twoKnownWords, outPrefix):
    oneInObj = open(oneInName, 'r')
    twoInObj = open(twoInName, 'r')

    knownPrefix = outPrefix + ".known"
    unkPrefix = outPrefix + ".unk"

    oneKnownOutObj = open(knownPrefix + ".lang1", 'w')
    twoKnownOutObj = open(knownPrefix + ".lang2", 'w')

    oneUnkOutObj = open(unkPrefix + ".lang1", 'w')
    twoUnkOutObj = open(unkPrefix + ".lang2", 'w')

    oneLine = oneInObj.readline()
    twoLine = twoInObj.readline()

    while len(oneLine) != 0 and len(twoLine) != 0:
        oneTuple = lineToTuple(oneLine)
        twoTuple = lineToTuple(twoLine)

        unksFound = False

        for word in oneTuple:
            if word not in oneKnownWords:
                unksFound = True
                break

        if not unksFound:
            for word in twoTuple:
                if word not in twoKnownWords:
                    unksFound = True
                    break

        if unksFound:
            oneUnkOutObj.write(oneLine)
            twoUnkOutObj.write(twoLine)
        else:
            oneKnownOutObj.write(oneLine)
            twoKnownOutObj.write(twoLine)

        oneLine = oneInObj.readline()
        twoLine = twoInObj.readline()

    assert len(oneLine) == 0 and len(twoLine) == 0 # corpus file lengths match

    oneKnownOutObj.close()
    twoKnownOutObj.close()

    oneUnkOutObj.close()
    twoUnkOutObj.close()

    oneInObj.close()
    twoInObj.close()

def main():
    if len(sys.argv) != 6:
        print "usage: %s oneInName twoInName oneKnownName twoKnownName outPrefix" % sys.argv[0]
        return

    oneKnownWords = fileToWordSet(sys.argv[3])
    twoKnownWords = fileToWordSet(sys.argv[4])

    createSubsets(sys.argv[1], sys.argv[2], oneKnownWords, twoKnownWords, sys.argv[5])

if __name__ == "__main__":
   main()
